@@ -499,6 +499,9 @@ class NLHandler {
     \rst
     Receives notification of a `common expression <mp::expr::COMMON_EXPR>`
     (defined variable) reference.
+    It's up to the Handler to check
+    that the CommonExpr has been provided
+    (according to the NL spec, actually before first use.)
     \endrst
    */
   Reference OnCommonExprRef(int expr_index) {
@@ -2131,7 +2134,8 @@ class NLProblemBuilder {
   /// Add variables
   void AddVariables(const NLHeader& h) {
     // Distinguish NL variable order
-    // See D.M.Gay, Hooking Your Solver to AMPL; and Writing .NL Files,
+    // See D.M.Gay, Hooking Your Solver to AMPL;
+    //     D.M.Gay, Writing .NL Files;
     // and, e.g.,
     // github.com/jump-dev/MathOptInterface.jl/blob/master/src/FileFormats/NL/README.md
     int k=0;                             // current block position
@@ -2293,6 +2297,9 @@ class NLProblemBuilder {
     return builder_.MakeVariable(var_index);
   }
 
+  /// It's up to the Handler to check
+  /// that the CommonExpr has been provided
+  /// (according to the NL spec, actually before first use.)
   Reference OnCommonExprRef(int expr_index) {
     return builder_.MakeCommonExpr(expr_index);
   }
@@ -2511,8 +2518,10 @@ public:
                fmt::CStringRef gen_name,
                std::size_t num_items);
 
-  /// Construct without reading (generic names can be provided)
-  NameProvider(fmt::CStringRef gen_name);
+  /// Construct without reading (generic names can be provided).
+  /// @param n2: second generic name,
+  ///   used from the 2nd index of name().
+  NameProvider(fmt::CStringRef gen_name, fmt::CStringRef n2="");
 
   /// Read names
   void ReadNames(fmt::CStringRef filename,
@@ -2522,15 +2531,19 @@ public:
   size_t number_read() const;
 
   /// Returns the name of the item at specified index.
-  fmt::StringRef name(std::size_t index);
+  /// @param i2: if >=0,
+  ///   from this index, generic name 2 is used
+  fmt::StringRef name(std::size_t index, std::size_t i2=-1);
 
   /// Return vector of names, length n.
   /// If number_read() < n, generic names are filled.
-  std::vector<std::string> get_names(size_t n);
+  /// @param i2: if >=0,
+  ///   from this index, generic name 2 is used
+  std::vector<std::string> get_names(size_t n, size_t i2=-1);
 
 private:
   std::vector<const char *> names_;
-  std::string gen_name_;
+  std::string gen_name_ {"_std_"}, gen_name_2_ {"_extra_"};
   internal::NameReader reader_;
   fmt::MemoryWriter writer_;
 };
