@@ -263,6 +263,7 @@ void SCIPvarCalcDistributionParameters(
    SCIP_Real             varlb,              /**< variable lower bound */
    SCIP_Real             varub,              /**< variable upper bound */
    SCIP_VARTYPE          vartype,            /**< type of the variable */
+   SCIP_IMPLINTTYPE      impltype,           /**< implied integral type of the variable */
    SCIP_Real*            mean,               /**< pointer to store mean value */
    SCIP_Real*            variance            /**< pointer to store the variance of the variable uniform distribution */
    )
@@ -286,7 +287,7 @@ void SCIPvarCalcDistributionParameters(
    else
    {
       /* if the variable is continuous, we assume a continuous uniform distribution, otherwise a discrete one */
-      if( vartype == SCIP_VARTYPE_CONTINUOUS )
+      if( vartype == SCIP_VARTYPE_CONTINUOUS && impltype == SCIP_IMPLINTTYPE_NONE )
          *variance = SQR(varub - varlb);
       else
          *variance = SQR(varub - varlb + 1.0) - 1.0;
@@ -528,7 +529,8 @@ void rowCalculateGauss(
                ++(*rowinfinitiesup);
          }
       }
-      SCIPvarCalcDistributionParameters(scip, colvarlb, colvarub, SCIPvarGetType(colvar), &varmean, &varvariance);
+      SCIPvarCalcDistributionParameters(scip, colvarlb, colvarub, SCIPvarGetType(colvar), SCIPvarGetImplType(colvar),
+            &varmean, &varvariance);
 
       /* actual values are updated; the contribution of the variable to mu is the arithmetic mean of its bounds */
       *mu += colval * varmean;
@@ -641,6 +643,7 @@ SCIP_RETCODE calcBranchScore(
    SCIP_Real meanup;           /* mean value of variable uniform distribution after branching up */
    SCIP_Real meandown;         /* mean value of variable uniform distribution after branching down*/
    SCIP_VARTYPE vartype;
+   SCIP_IMPLINTTYPE impltype;
    int ncolrows;
    int i;
 
@@ -663,11 +666,12 @@ SCIP_RETCODE calcBranchScore(
    varub = SCIPvarGetUbLocal(var);
    assert(SCIPisFeasLT(scip, varlb, varub));
    vartype = SCIPvarGetType(var);
+   impltype = SCIPvarGetImplType(var);
 
    /* calculate mean and variance of variable uniform distribution before and after branching */
    currentmean = 0.0;
    squaredbounddiff = 0.0;
-   SCIPvarCalcDistributionParameters(scip, varlb, varub, vartype, &currentmean, &squaredbounddiff);
+   SCIPvarCalcDistributionParameters(scip, varlb, varub, vartype, impltype, &currentmean, &squaredbounddiff);
 
    newlb = SCIPfeasCeil(scip, lpsolval);
    newub = SCIPfeasFloor(scip, lpsolval);
@@ -675,12 +679,12 @@ SCIP_RETCODE calcBranchScore(
    /* calculate the variable's uniform distribution after branching up and down, respectively. */
    squaredbounddiffup = 0.0;
    meanup = 0.0;
-   SCIPvarCalcDistributionParameters(scip, newlb, varub, vartype, &meanup, &squaredbounddiffup);
+   SCIPvarCalcDistributionParameters(scip, newlb, varub, vartype, impltype, &meanup, &squaredbounddiffup);
 
    /* calculate the distribution mean and variance for a variable with finite lower bound */
    squaredbounddiffdown = 0.0;
    meandown = 0.0;
-   SCIPvarCalcDistributionParameters(scip, varlb, newub, vartype, &meandown, &squaredbounddiffdown);
+   SCIPvarCalcDistributionParameters(scip, varlb, newub, vartype, impltype, &meandown, &squaredbounddiffdown);
 
    /* initialize the variable's up and down score */
    *upscore = 0.0;
@@ -928,6 +932,7 @@ SCIP_RETCODE varProcessBoundChanges(
    SCIP_Real oldub;
    SCIP_Real newub;
    SCIP_VARTYPE vartype;
+   SCIP_IMPLINTTYPE impltype;
    int ncolrows;
    int r;
    int varindex;
@@ -967,8 +972,9 @@ SCIP_RETCODE varProcessBoundChanges(
    oldmean = 0.0;
    newmean = 0.0;
    vartype = SCIPvarGetType(var);
-   SCIPvarCalcDistributionParameters(scip, oldlb, oldub, vartype, &oldmean, &oldvariance);
-   SCIPvarCalcDistributionParameters(scip, newlb, newub, vartype, &newmean, &newvariance);
+   impltype = SCIPvarGetImplType(var);
+   SCIPvarCalcDistributionParameters(scip, oldlb, oldub, vartype, impltype, &oldmean, &oldvariance);
+   SCIPvarCalcDistributionParameters(scip, newlb, newub, vartype, impltype, &newmean, &newvariance);
 
    /* loop over all rows of this variable and update activity distribution */
    for( r = 0; r < ncolrows; ++r )
@@ -1360,7 +1366,7 @@ SCIP_RETCODE SCIPincludeBranchruleDistribution(
 
    /* include branching rule */
    SCIP_CALL( SCIPincludeBranchruleBasic(scip, &branchrule, BRANCHRULE_NAME, BRANCHRULE_DESC, BRANCHRULE_PRIORITY, BRANCHRULE_MAXDEPTH,
-	 BRANCHRULE_MAXBOUNDDIST, branchruledata) );
+         BRANCHRULE_MAXBOUNDDIST, branchruledata) );
 
    assert(branchrule != NULL);
    SCIP_CALL( SCIPsetBranchruleCopy(scip, branchrule, branchCopyDistribution) );
