@@ -264,7 +264,8 @@ SCIP_RETCODE initConcsolver(
 
    /* we force the copying of symmetry constraints that may have been detected during a central presolving step;
     * otherwise, the copy may become invalid */
-   if( SCIPsetBoolParam(scip, "constraints/orbitope/forceconscopy", TRUE) != SCIP_OKAY
+   if( SCIPsetBoolParam(scip, "constraints/orbitope_full/forceconscopy", TRUE) != SCIP_OKAY
+      || SCIPsetBoolParam(scip, "constraints/orbitope_pp/forceconscopy", TRUE) != SCIP_OKAY
       || SCIPsetBoolParam(scip, "constraints/orbisack/forceconscopy", TRUE) != SCIP_OKAY
       || SCIPsetBoolParam(scip, "constraints/symresack/forceconscopy", TRUE) != SCIP_OKAY )
    {
@@ -803,7 +804,7 @@ SCIP_DECL_CONCSOLVERSYNCREAD(concsolverScipSyncRead)
       /* bound is better so incremented counters for statistics and pass it to the sync propagator */
       ++(*ntighterbnds);
 
-      if( SCIPvarGetType(var) <= SCIP_VARTYPE_INTEGER )
+      if( SCIPvarIsNonimpliedIntegral(var) )
          ++(*ntighterintbnds);
 
       SCIP_CALL( SCIPaddConcurrentBndchg(data->solverscip, var, newbound, boundtype) );
