@@ -860,6 +860,7 @@ SCIP_RETCODE SCIPdialogCreate(
    (*dialog)->subdialogssize = 0;
    (*dialog)->nuses = 0;
    (*dialog)->dialogdata = dialogdata;
+   (*dialog)->hidden = FALSE;
 
    /* capture dialog */
    SCIPdialogCapture(*dialog);
@@ -934,6 +935,24 @@ SCIP_RETCODE SCIPdialogRelease(
    return SCIP_OKAY;
 }
 
+/** is dialog hidden */
+SCIP_Bool SCIPdialogIsHidden(
+   SCIP_DIALOG*          dialog              /**< dialog */
+   )
+{
+   assert(dialog != NULL);
+   return dialog->hidden;
+}
+
+/** set dialog to be hidden */
+void SCIPdialogSetHidden(
+   SCIP_DIALOG*          dialog              /**< dialog */
+   )
+{
+   assert(dialog != NULL);
+   dialog->hidden = TRUE;
+}
+
 /** executes dialog */
 SCIP_RETCODE SCIPdialogExec(
    SCIP_DIALOG*          dialog,             /**< dialog */
@@ -1070,7 +1089,7 @@ int SCIPdialogFindEntry(
 /** displays the dialog's menu */
 SCIP_RETCODE SCIPdialogDisplayMenu(
    SCIP_DIALOG*          dialog,             /**< dialog */
-   SCIP*                 scip                /**< SCIP data structure */   
+   SCIP*                 scip                /**< SCIP data structure */
    )
 {
    int i;
@@ -1089,7 +1108,7 @@ SCIP_RETCODE SCIPdialogDisplayMenu(
    /* display the dialog's menu options */
    for( i = 0; i < dialog->nsubdialogs; ++i )
    {
-      if( !SCIPdialogIsSubmenu(dialog->subdialogs[i]) )
+      if( !SCIPdialogIsSubmenu(dialog->subdialogs[i]) && ! SCIPdialogIsHidden(dialog->subdialogs[i]) )
       {
          SCIP_CALL( SCIPdialogDisplayMenuEntry(dialog->subdialogs[i], scip) );
       }
@@ -1104,7 +1123,7 @@ SCIP_RETCODE SCIPdialogDisplayMenu(
 /** displays the entry for the dialog in it's parent's menu */
 SCIP_RETCODE SCIPdialogDisplayMenuEntry(
    SCIP_DIALOG*          dialog,             /**< dialog */
-   SCIP*                 scip                /**< SCIP data structure */   
+   SCIP*                 scip                /**< SCIP data structure */
    )
 {
    char name[SCIP_MAXSTRLEN];
@@ -1138,7 +1157,7 @@ SCIP_RETCODE SCIPdialogDisplayMenuEntry(
 /** displays all dialog entries with names starting with the given "entryname" */
 SCIP_RETCODE SCIPdialogDisplayCompletions(
    SCIP_DIALOG*          dialog,             /**< dialog */
-   SCIP*                 scip,               /**< SCIP data structure */   
+   SCIP*                 scip,               /**< SCIP data structure */
    const char*           entryname           /**< name of the dialog entry to find */
    )
 {
