@@ -342,7 +342,8 @@ void rowCalculateGauss(
                ++(*rowinfinitiesup);
          }
       }
-      SCIPvarCalcDistributionParameters(scip, colvarlb, colvarub, SCIPvarGetType(colvar), &varmean, &varvariance);
+      SCIPvarCalcDistributionParameters(scip, colvarlb, colvarub, SCIPvarGetType(colvar), SCIPvarGetImplType(colvar),
+            &varmean, &varvariance);
 
       /* actual values are updated; the contribution of the variable to mu is the arithmetic mean of its bounds */
       *mu += colval * varmean;
@@ -384,6 +385,7 @@ SCIP_RETCODE calcBranchScore(
    SCIP_Real meanup;           /* mean value of variable uniform distribution after branching up */
    SCIP_Real meandown;         /* mean value of variable uniform distribution after branching down*/
    SCIP_VARTYPE vartype;
+   SCIP_IMPLINTTYPE impltype;
    int ncolrows;
    int i;
 
@@ -404,11 +406,12 @@ SCIP_RETCODE calcBranchScore(
    varub = SCIPvarGetUbLocal(var);
    assert(varub - varlb > 0.5);
    vartype = SCIPvarGetType(var);
+   impltype = SCIPvarGetImplType(var);
 
    /* calculate mean and variance of variable uniform distribution before and after branching */
    currentmean = 0.0;
    squaredbounddiff = 0.0;
-   SCIPvarCalcDistributionParameters(scip, varlb, varub, vartype, &currentmean, &squaredbounddiff);
+   SCIPvarCalcDistributionParameters(scip, varlb, varub, vartype, impltype, &currentmean, &squaredbounddiff);
 
    /* unfixed binary variables may have an integer solution value in the LP solution, eg, at the presence of indicator constraints */
    if( !SCIPvarIsBinary(var) )
@@ -425,12 +428,12 @@ SCIP_RETCODE calcBranchScore(
    /* calculate the variable's uniform distribution after branching up and down, respectively. */
    squaredbounddiffup = 0.0;
    meanup = 0.0;
-   SCIPvarCalcDistributionParameters(scip, newlb, varub, vartype, &meanup, &squaredbounddiffup);
+   SCIPvarCalcDistributionParameters(scip, newlb, varub, vartype, impltype, &meanup, &squaredbounddiffup);
 
    /* calculate the distribution mean and variance for a variable with finite lower bound */
    squaredbounddiffdown = 0.0;
    meandown = 0.0;
-   SCIPvarCalcDistributionParameters(scip, varlb, newub, vartype, &meandown, &squaredbounddiffdown);
+   SCIPvarCalcDistributionParameters(scip, varlb, newub, vartype, impltype, &meandown, &squaredbounddiffdown);
 
    /* initialize the variable's up and down score */
    *upscore = 0.0;
@@ -703,6 +706,7 @@ SCIP_RETCODE varProcessBoundChanges(
    SCIP_Real oldub;
    SCIP_Real newub;
    SCIP_VARTYPE vartype;
+   SCIP_IMPLINTTYPE impltype;
    int ncolrows;
    int r;
    int varindex;
@@ -740,8 +744,9 @@ SCIP_RETCODE varProcessBoundChanges(
    oldmean = 0.0;
    newmean = 0.0;
    vartype = SCIPvarGetType(var);
-   SCIPvarCalcDistributionParameters(scip, oldlb, oldub, vartype, &oldmean, &oldvariance);
-   SCIPvarCalcDistributionParameters(scip, newlb, newub, vartype, &newmean, &newvariance);
+   impltype = SCIPvarGetImplType(var);
+   SCIPvarCalcDistributionParameters(scip, oldlb, oldub, vartype, impltype, &oldmean, &oldvariance);
+   SCIPvarCalcDistributionParameters(scip, newlb, newub, vartype, impltype, &newmean, &newvariance);
 
    /* loop over all rows of this variable and update activity distribution */
    for( r = 0; r < ncolrows; ++r )
@@ -989,7 +994,7 @@ SCIP_DECL_HEUREXEC(heurExecDistributiondiving)
       return SCIP_OKAY;
 
    /* terminate if there are no integer variables (note that, e.g., SOS1 variables may be present) */
-   if( SCIPgetNBinVars(scip) + SCIPgetNIntVars(scip) == 0 )
+   if( SCIPgetNContVars(scip) + SCIPgetNContImplVars(scip) == SCIPgetNVars(scip) )
       return SCIP_OKAY;
 
    /* select and store the scoring parameter for this call of the heuristic */
@@ -1079,6 +1084,9 @@ SCIP_RETCODE SCIPincludeHeurDistributiondiving(
 
    assert(heur != NULL);
 
+   /* primal heuristic is safe to use in exact solving mode */
+   SCIPheurMarkExact(heur);
+
    /* set non-NULL pointers to callback methods */
    SCIP_CALL( SCIPsetHeurCopy(scip, heur, heurCopyDistributiondiving) );
    SCIP_CALL( SCIPsetHeurFree(scip, heur, heurFreeDistributiondiving) );
