@@ -197,7 +197,7 @@ SCIP_RETCODE addLocalbranchingConstraintAndObjcutoff(
          consvals[nconsvars] = 1.0;
 
       consvars[nconsvars] = subvars[i];
-      assert( SCIPvarGetType(consvars[nconsvars]) == SCIP_VARTYPE_BINARY );
+      assert( SCIPvarGetType(consvars[nconsvars]) == SCIP_VARTYPE_BINARY && !SCIPvarIsImpliedIntegral(consvars[nconsvars]) );
 
       ++nconsvars;
    }
@@ -623,6 +623,9 @@ SCIP_RETCODE SCIPincludeHeurLocalbranching(
 
    assert(heur != NULL);
 
+   /* primal heuristic is safe to use in exact solving mode */
+   SCIPheurMarkExact(heur);
+
    /* set non-NULL pointers to callback methods */
    SCIP_CALL( SCIPsetHeurCopy(scip, heur, heurCopyLocalbranching) );
    SCIP_CALL( SCIPsetHeurFree(scip, heur, heurFreeLocalbranching) );
