@@ -40,8 +40,10 @@
 #include "scip/pub_misc.h"
 #include "scip/pub_var.h"
 #include "scip/scip_branch.h"
+#include "scip/scip_certificate.h"
 #include "scip/scip_cons.h"
 #include "scip/scip_copy.h"
+#include "scip/scip_exact.h"
 #include "scip/scip_general.h"
 #include "scip/scip_heur.h"
 #include "scip/scip_lp.h"
@@ -182,9 +184,6 @@ SCIP_DECL_HEUREXIT(heurExitLocks) /*lint --e{715}*/
    return SCIP_OKAY;
 }
 
-#define heurInitsolLocks NULL
-#define heurExitsolLocks NULL
-
 /** apply fix-and-propagate scheme based on variable locks
  *
  *  @note probing mode of SCIP needs to be enabled before
@@ -589,7 +588,7 @@ SCIP_RETCODE SCIPapplyLockFixings(
                for( w = ncols - 1; w >= 0; --w  )
                {
                   colvar = SCIPcolGetVar(cols[w]);
-                  if( SCIPvarGetType(colvar) == SCIP_VARTYPE_BINARY && colvar != var )
+                  if( SCIPvarGetType(colvar) == SCIP_VARTYPE_BINARY && !SCIPvarIsImpliedIntegral(colvar) && colvar != var )
                   {
                      assert(sortvars[varpos[SCIPvarGetProbindex(colvar)]] == colvar);
                      pos = varpos[SCIPvarGetProbindex(colvar)];
@@ -688,8 +687,11 @@ SCIP_DECL_HEUREXEC(heurExecLocks)
    {
       SCIP_CALL( SCIPconstructLP(scip, &cutoff) );
 
-      /* manually cut off the node if the LP construction detected infeasibility (heuristics cannot return such a result) */
-      if( cutoff )
+      /* manually cut off the node if the LP construction detected infeasibility (heuristics cannot return such a
+       * result); the cutoff result is safe to use in exact solving mode, but we don't have enough information to
+       * give a certificate for the cutoff
+       */
+      if( cutoff && !SCIPisCertified(scip) )
       {
          SCIP_CALL( SCIPcutoffNode(scip, SCIPgetCurrentNode(scip)) );
          return SCIP_OKAY;
@@ -1083,18 +1085,28 @@ SCIP_RETCODE SCIPincludeHeurLocks(
    )
 {
    SCIP_HEURDATA* heurdata;
+   SCIP_HEUR* heur;
 
    /* create primal heuristic data */
    SCIP_CALL( SCIPallocBlockMemory(scip, &heurdata) );
 
    /* include primal heuristic */
-   SCIP_CALL( SCIPincludeHeur(scip, HEUR_NAME, HEUR_DESC, HEUR_DISPCHAR, HEUR_PRIORITY, HEUR_FREQ, HEUR_FREQOFS,
-         HEUR_MAXDEPTH, HEUR_TIMING, HEUR_USESSUBSCIP,
-         heurCopyLocks,
-         heurFreeLocks, heurInitLocks, heurExitLocks,
-         heurInitsolLocks, heurExitsolLocks, heurExecLocks,
-         heurdata) );
+   SCIP_CALL( SCIPincludeHeurBasic(scip, &heur,
+         HEUR_NAME, HEUR_DESC, HEUR_DISPCHAR, HEUR_PRIORITY, HEUR_FREQ, HEUR_FREQOFS,
+         HEUR_MAXDEPTH, HEUR_TIMING, HEUR_USESSUBSCIP, heurExecLocks, heurdata) );
 
+   assert(heur != NULL);
+
+   /* primal heuristic is safe to use in exact solving mode */
+   SCIPheurMarkExact(heur);
+
+   /* set non-NULL pointers to callback methods */
+   SCIP_CALL( SCIPsetHeurCopy(scip, heur, heurCopyLocks) );
+   SCIP_CALL( SCIPsetHeurFree(scip, heur, heurFreeLocks) );
+   SCIP_CALL( SCIPsetHeurInit(scip, heur, heurInitLocks) );
+   SCIP_CALL( SCIPsetHeurExit(scip, heur, heurExitLocks) );
+
+   /* add locks primal heuristic parameters */
    SCIP_CALL( SCIPaddIntParam(scip, "heuristics/" HEUR_NAME "/maxproprounds",
          "maximum number of propagation rounds to be performed in each propagation call (-1: no limit, -2: parameter settings)",
          &heurdata->maxproprounds, TRUE, DEFAULT_MAXPROPROUNDS, -2, INT_MAX, NULL, NULL) );
