@@ -319,7 +319,7 @@ SCIP_RETCODE setupAndSolveSubscipMutation(
          cutoff = (1 + heurdata->minimprove) * SCIPgetUpperbound(scip);
    }
    cutoff = MIN(upperbound, cutoff);
-   SCIP_CALL(SCIPsetObjlimit(subscip, cutoff));
+   SCIP_CALL( SCIPsetObjlimit(subscip, cutoff) );
 
    /* solve the subproblem
     *
@@ -564,6 +564,9 @@ SCIP_RETCODE SCIPincludeHeurMutation(
 
    assert(heur != NULL);
 
+   /* primal heuristic is safe to use in exact solving mode */
+   SCIPheurMarkExact(heur);
+
    /* set non-NULL pointers to callback methods */
    SCIP_CALL( SCIPsetHeurCopy(scip, heur, heurCopyMutation) );
    SCIP_CALL( SCIPsetHeurFree(scip, heur, heurFreeMutation) );
