@@ -26,7 +26,7 @@
  * @ingroup DEFPLUGINS_HEUR
  * @brief   NLP local search primal heuristic using sub-SCIPs
  * @author  Stefan Vigerske
- * 
+ *
  * @todo reconstruct sub-SCIP if problem has changed
  */
 
@@ -124,6 +124,7 @@ struct SCIP_HeurData
    int                   iterinit;           /**< number of iterations used for initial NLP solves */
    int                   ninitsolves;        /**< number of successful NLP solves until switching to iterlimit guess and using success rate */
    int                   itermin;            /**< minimal number of iterations for NLP solves */
+   SCIP_HEURTIMING       inittiming;         /**< initial heuristic timing */
 };
 
 
@@ -131,36 +132,6 @@ struct SCIP_HeurData
  * Local methods
  */
 
-/** indicates whether the heuristic should be running, i.e., whether we expect something nonlinear after fixing all discrete variables */
-static
-SCIP_RETCODE runHeuristic(
-   SCIP*                 scip,               /**< SCIP data structure */
-   SCIP_Bool*            runheur             /**< buffer to store whether to run heuristic */
-   )
-{
-   assert(scip != NULL);
-   assert(runheur != NULL);
-
-   /* do not run heuristic if no NLP solver is available */
-   if( SCIPgetNNlpis(scip) <= 0 )
-   {
-      *runheur = FALSE;
-      return SCIP_OKAY;
-   }
-
-   /* do not run heuristic if no NLP */
-   if( !SCIPisNLPConstructed(scip) )
-   {
-      *runheur = FALSE;
-      return SCIP_OKAY;
-   }
-
-   /* do not run heuristic if no continuous nonlinear variables in NLP */
-   SCIP_CALL( SCIPhasNLPContinuousNonlinearity(scip, runheur) );
-
-   return SCIP_OKAY;
-}
-
 /** free sub-SCIP data structure */
 static
 SCIP_RETCODE freeSubSCIP(
@@ -258,6 +229,7 @@ SCIP_RETCODE createSubSCIP(
          TRUE,  /* eventhandler */
          TRUE,  /* nodeselectors (SCIP gives an error if there is none) */
          FALSE, /* branchrules */
+         FALSE, /* iis */
          TRUE,  /* displays */
          FALSE, /* tables */
          FALSE, /* dialogs */
@@ -396,7 +368,8 @@ SCIP_RETCODE createSubSCIP(
    SCIP_CALL( SCIPresetParam(heurdata->subscip, "limits/dual") );
 
    /* we remember here which way (continuous or not) we went, in case all binary and integer vars get fixed in root */
-   heurdata->continuous = SCIPgetNBinVars(heurdata->subscip) == 0 && SCIPgetNIntVars(heurdata->subscip) == 0;
+   heurdata->continuous = (SCIPgetNContVars(heurdata->subscip) + SCIPgetNContImplVars(heurdata->subscip)
+         == SCIPgetNVars(heurdata->subscip));
    if( !heurdata->continuous )
    {
       /* set presolve maxrounds and emphasis; always disable components presolver
@@ -1302,7 +1275,7 @@ SCIP_RETCODE forbidFixation(
       /* allocate memory for constraint variables */
       SCIP_CALL( SCIPallocBufferArray(scip, &consvars, nsubbinvars) );
 
-      /* get fixations of discrete variables 
+      /* get fixations of discrete variables
        * to be sure, we take the values that were put into the subCIP before
        */
       nconsvars = 0;
@@ -1355,7 +1328,7 @@ SCIP_RETCODE forbidFixation(
       SCIP_CALL( SCIPallocBufferArray(scip, &boundtypes, nsubbinvars + 2*nsubintvars) );
       SCIP_CALL( SCIPallocBufferArray(scip, &bounds,     nsubbinvars + 2*nsubintvars) );
 
-      /* get fixations of discrete variables 
+      /* get fixations of discrete variables
        * to be sure, we take the values that were put into the subCIP before
        */
       nconsvars = 0;
@@ -1373,7 +1346,8 @@ SCIP_RETCODE forbidFixation(
          fixval = SCIPvarGetLbGlobal(subvar);
          assert(fixval == SCIPvarGetUbGlobal(subvar)); /* variable should be fixed in sub-SCIP */   /*lint !e777*/
          assert(SCIPceil(scip, fixval - 0.5) == fixval); /* we have rounded values before fixing */ /*lint !e777*/
-         assert(SCIPvarGetType(var) != SCIP_VARTYPE_BINARY || SCIPvarGetLbGlobal(var) == fixval || SCIPvarGetUbGlobal(var) == fixval); /* for binaries, the fixval should be either 0.0 or 1.0 */  /*lint !e777*/
+         assert(SCIPvarGetType(var) != SCIP_VARTYPE_BINARY || SCIPvarIsImpliedIntegral(var)
+               || SCIPvarGetLbGlobal(var) == fixval || SCIPvarGetUbGlobal(var) == fixval); /* for binaries, the fixval should be either 0.0 or 1.0 */  /*lint !e777*/
 
          if( SCIPvarGetLbGlobal(var) < fixval )
          {
@@ -1494,12 +1468,24 @@ SCIP_DECL_HEURINIT(heurInitSubNlp)
 static
 SCIP_DECL_HEURINITSOL(heurInitsolSubNlp)
 {
+   SCIP_HEURDATA* heurdata;
+
    assert(scip != NULL);
    assert(heur != NULL);
 
+   /* get heuristic data */
+   heurdata = SCIPheurGetData(heur);
+   assert(heurdata != NULL);
+
+   /* store subnlp timing */
+   heurdata->inittiming = SCIPheurGetTimingmask(heur);
+
+   /* disable subnlp heuristic */
+   if( !SCIPisNLPConstructed(scip) || SCIPgetNNlpis(scip) == 0 )
+      SCIPheurSetTimingmask(heur, SCIP_HEURTIMING_NONE);
    /* if the heuristic is called at the root node, we want to be called directly after the initial root LP solve */
-   if( SCIPheurGetFreqofs(heur) == 0 )
-      SCIPheurSetTimingmask(heur, SCIP_HEURTIMING_DURINGLPLOOP | HEUR_TIMING);
+   else if( SCIPheurGetFreqofs(heur) == 0 )
+      SCIPheurSetTimingmask(heur, heurdata->inittiming | SCIP_HEURTIMING_DURINGLPLOOP);
 
    return SCIP_OKAY;
 }
@@ -1509,10 +1495,11 @@ static
 SCIP_DECL_HEUREXITSOL(heurExitsolSubNlp)
 {
    SCIP_HEURDATA* heurdata;
+
    assert(scip != NULL);
    assert(heur != NULL);
 
-   /* get heuristic's data */  
+   /* get heuristic data */
    heurdata = SCIPheurGetData(heur);
    assert(heurdata != NULL);
 
@@ -1528,7 +1515,8 @@ SCIP_DECL_HEUREXITSOL(heurExitsolSubNlp)
       SCIP_CALL( SCIPfreeSol(scip, &heurdata->startcand) );
    }
 
-   SCIPheurSetTimingmask(heur, HEUR_TIMING);
+   /* reset subnlp timing */
+   SCIPheurSetTimingmask(heur, heurdata->inittiming);
 
    return SCIP_OKAY;
 }
@@ -1539,15 +1527,28 @@ static
 SCIP_DECL_HEUREXEC(heurExecSubNlp)
 {  /*lint --e{666,715}*/
    SCIP_HEURDATA* heurdata;
-   SCIP_Bool      runheur;
-   SCIP_Real      itercontingent;
+   SCIP_Bool runheur;
+   SCIP_Real itercontingent;
 
    assert(scip != NULL);
    assert(heur != NULL);
+   assert(SCIPisNLPConstructed(scip));
+   assert(SCIPgetNNlpis(scip) >= 1);
 
    /* obviously, we did not do anything yet */
    *result = SCIP_DIDNOTRUN;
 
+   /* before we run the heuristic for the first time, check whether we want to run the heuristic at all */
+   if( SCIPheurGetNCalls(heur) == 0 )
+   {
+      SCIP_CALL( SCIPhasNLPContinuousNonlinearity(scip, &runheur) );
+      if( !runheur )
+      {
+         SCIPheurSetTimingmask(heur, SCIP_HEURTIMING_NONE);
+         return SCIP_OKAY;
+      }
+   }
+
    /* get heuristic's data */
    heurdata = SCIPheurGetData(heur);
    assert(heurdata != NULL);
@@ -1560,14 +1561,6 @@ SCIP_DECL_HEUREXEC(heurExecSubNlp)
    if( heurdata->subscip == NULL && heurdata->keepcopy && heurdata->triedsetupsubscip )
       return SCIP_OKAY;
 
-   /* before we run the heuristic for the first time, check whether we want to run the heuristic at all */
-   if( SCIPheurGetNCalls(heur) == 0 )
-   {
-      SCIP_CALL( runHeuristic(scip, &runheur) );
-      if( !runheur )
-         return SCIP_OKAY;
-   }
-
    if( heurdata->startcand == NULL )
    {
       /* if no start candidate is given, we consider the LP solution of the current node */
@@ -1576,7 +1569,7 @@ SCIP_DECL_HEUREXEC(heurExecSubNlp)
       if( nodeinfeasible )
          return SCIP_OKAY;
 
-      /* at least if we are not called the first time, we call the heuristic only if an optimal LP solution is available 
+      /* at least if we are not called the first time, we call the heuristic only if an optimal LP solution is available
        * if we are called the first time and the LP is unbounded, then we are quite desperate and still give the NLP a try
        */
       if( SCIPgetLPSolstat(scip) != SCIP_LPSOLSTAT_OPTIMAL )
@@ -1679,6 +1672,7 @@ SCIP_RETCODE SCIPincludeHeurSubNlp(
    /* create Nlp primal heuristic data */
    SCIP_CALL( SCIPallocBlockMemory(scip, &heurdata) );
    BMSclearMemory(heurdata);
+   heurdata->inittiming = HEUR_TIMING;
 
    /* include variable event handler */
    heurdata->eventhdlr = NULL;
@@ -1937,7 +1931,7 @@ SCIP_RETCODE SCIPapplyHeurSubNlp(
 }
 
 /** updates the starting point for the NLP heuristic
- * 
+ *
  * Is called by a constraint handler that handles nonlinear constraints when a check on feasibility of a solution fails.
  */
 SCIP_RETCODE SCIPupdateStartpointHeurSubNlp(
@@ -1959,6 +1953,10 @@ SCIP_RETCODE SCIPupdateStartpointHeurSubNlp(
    if( SCIPgetStage(scip) != SCIP_STAGE_SOLVING )
       return SCIP_OKAY;
 
+   /* only update starting point if an NLP relaxation has been constructed */
+   if( !SCIPisNLPConstructed(scip) || SCIPgetNNlpis(scip) == 0 )
+      return SCIP_OKAY;
+
    heurdata = SCIPheurGetData(heur);
    assert(heurdata != NULL);
 
@@ -1970,7 +1968,7 @@ SCIP_RETCODE SCIPupdateStartpointHeurSubNlp(
          return SCIP_OKAY;
       if( SCIPheurGetFreq(heur) < 0 )
          return SCIP_OKAY;
-      SCIP_CALL( runHeuristic(scip, &runheur) );
+      SCIP_CALL( SCIPhasNLPContinuousNonlinearity(scip, &runheur) );
       if( !runheur )
          return SCIP_OKAY;
    }
