@@ -588,6 +588,9 @@ SCIP_RETCODE SCIPincludeHeurTrustregion(
 
    assert(heur != NULL);
 
+   /* primal heuristic is safe to use in exact solving mode */
+   SCIPheurMarkExact(heur);
+
    /* set non-NULL pointers to callback methods */
    SCIP_CALL( SCIPsetHeurCopy(scip, heur, heurCopyTrustregion) );
    SCIP_CALL( SCIPsetHeurFree(scip, heur, heurFreeTrustregion) );
