@@ -321,7 +321,7 @@ SCIP_RETCODE setupAndSolveSubscip(
       ub = SCIPvarGetUbGlobal(subvars[i]);
       inf = SCIPinfinity(subscip);
 
-      /* adjust infinite bounds in order to avoid that variables with non-zero objective 
+      /* adjust infinite bounds in order to avoid that variables with non-zero objective
        * get fixed to infinite value in zeroobj subproblem
        */
       if( SCIPisInfinity(subscip, ub ) )
@@ -572,6 +572,9 @@ SCIP_RETCODE SCIPincludeHeurZeroobj(
          HEUR_MAXDEPTH, HEUR_TIMING, HEUR_USESSUBSCIP, heurExecZeroobj, heurdata) );
    assert(heur != NULL);
 
+   /* primal heuristic is safe to use in exact solving mode */
+   SCIPheurMarkExact(heur);
+
    /* set non-NULL pointers to callback methods */
    SCIP_CALL( SCIPsetHeurCopy(scip, heur, heurCopyZeroobj) );
    SCIP_CALL( SCIPsetHeurFree(scip, heur, heurFreeZeroobj) );
