@@ -43,15 +43,18 @@
 #include "scip/tree.h"
 #include "scip/reopt.h"
 #include "scip/lp.h"
+#include "scip/primal.h"
 #include "scip/scip.h"
 #include "scip/nodesel.h"
 #include "scip/pub_message.h"
 #include "scip/pub_misc.h"
+#include "scip/event.h"
 
-#include "scip/struct_nodesel.h"
 #include "scip/struct_scip.h"
+#include "scip/struct_nodesel.h"
+#include "scip/struct_event.h"
 
-/* 
+/*
  * node priority queue methods
  */
 
@@ -143,8 +146,8 @@ SCIP_RETCODE SCIPnodepqFree(
    BMS_BLKMEM*           blkmem,             /**< block memory buffers */
    SCIP_SET*             set,                /**< global SCIP settings */
    SCIP_STAT*            stat,               /**< problem statistics */
-   SCIP_EVENTFILTER*     eventfilter,        /**< event filter for global (not variable dependent) events */
    SCIP_EVENTQUEUE*      eventqueue,         /**< event queue */
+   SCIP_EVENTFILTER*     eventfilter,        /**< global event filter */
    SCIP_TREE*            tree,               /**< branch and bound tree */
    SCIP_LP*              lp                  /**< current LP data */
    )
@@ -153,7 +156,7 @@ SCIP_RETCODE SCIPnodepqFree(
    assert(*nodepq != NULL);
 
    /* free the nodes of the queue */
-   SCIP_CALL( SCIPnodepqClear(*nodepq, blkmem, set, stat, eventfilter, eventqueue, tree, lp) );
+   SCIP_CALL( SCIPnodepqClear(*nodepq, blkmem, set, stat, eventqueue, eventfilter, tree, lp) );
 
    /* free the queue data structure */
    SCIPnodepqDestroy(nodepq);
@@ -167,8 +170,8 @@ SCIP_RETCODE SCIPnodepqClear(
    BMS_BLKMEM*           blkmem,             /**< block memory buffers */
    SCIP_SET*             set,                /**< global SCIP settings */
    SCIP_STAT*            stat,               /**< problem statistics */
-   SCIP_EVENTFILTER*     eventfilter,        /**< event filter for global (not variable dependent) events */
    SCIP_EVENTQUEUE*      eventqueue,         /**< event queue */
+   SCIP_EVENTFILTER*     eventfilter,        /**< global event filter */
    SCIP_TREE*            tree,               /**< branch and bound tree */
    SCIP_LP*              lp                  /**< current LP data */
    )
@@ -191,7 +194,7 @@ SCIP_RETCODE SCIPnodepqClear(
          assert(nodepq->slots[i] != NULL);
          assert(SCIPnodeGetType(nodepq->slots[i]) == SCIP_NODETYPE_LEAF);
 
-         SCIP_CALL( SCIPnodeFree(&nodepq->slots[i], blkmem, set, stat, eventfilter, eventqueue, tree, lp) );
+         SCIP_CALL( SCIPnodeFree(&nodepq->slots[i], blkmem, set, stat, eventqueue, eventfilter, tree, lp) );
       }
    }
 
@@ -287,7 +290,6 @@ SCIP_RETCODE SCIPnodepqInsert(
    SCIP_NODE** slots;
    int* bfsposs;
    int* bfsqueue;
-   SCIP_Real lowerbound;
    int pos;
    int bfspos;
 
@@ -319,19 +321,34 @@ SCIP_RETCODE SCIPnodepqInsert(
    slots[pos] = node;
 
    /* insert the final position into the bfs index queue */
-   lowerbound = SCIPnodeGetLowerbound(node);
    bfspos = nodepq->len-1;
-   while( bfspos > 0 && lowerbound < SCIPnodeGetLowerbound(slots[bfsqueue[PQ_PARENT(bfspos)]]) )
+   if( set->exact_enable )
    {
-      bfsqueue[bfspos] = bfsqueue[PQ_PARENT(bfspos)];
-      bfsposs[bfsqueue[bfspos]] = bfspos;
-      bfspos = PQ_PARENT(bfspos);
+      SCIP_RATIONAL* lowerbound = SCIPnodeGetLowerboundExact(node);
+      while( bfspos > 0 && SCIPrationalIsLT(lowerbound, SCIPnodeGetLowerboundExact(slots[bfsqueue[PQ_PARENT(bfspos)]])) )
+      {
+         bfsqueue[bfspos] = bfsqueue[PQ_PARENT(bfspos)];
+         bfsposs[bfsqueue[bfspos]] = bfspos;
+         bfspos = PQ_PARENT(bfspos);
+      }
+
+      SCIPrationalDebugMessage("inserted node %p[%q] at pos %d and bfspos %d of node queue\n", (void*)node, lowerbound, pos, bfspos);
+   }
+   else
+   {
+      SCIP_Real lowerbound = SCIPnodeGetLowerbound(node);
+      while( bfspos > 0 && lowerbound < SCIPnodeGetLowerbound(slots[bfsqueue[PQ_PARENT(bfspos)]]) )
+      {
+         bfsqueue[bfspos] = bfsqueue[PQ_PARENT(bfspos)];
+         bfsposs[bfsqueue[bfspos]] = bfspos;
+         bfspos = PQ_PARENT(bfspos);
+      }
+
+      SCIPsetDebugMsg(set, "inserted node %p[%g] at pos %d and bfspos %d of node queue\n", (void*)node, lowerbound, pos, bfspos);
    }
    bfsqueue[bfspos] = pos;
    bfsposs[pos] = bfspos;
 
-   SCIPsetDebugMsg(set, "inserted node %p[%g] at pos %d and bfspos %d of node queue\n", (void*)node, lowerbound, pos, bfspos);
-
    return SCIP_OKAY;
 }
 
@@ -446,44 +463,84 @@ SCIP_Bool nodepqDelPos(
    bfsparentfelldown = FALSE;
    if( freebfspos < nodepq->len )
    {
-      SCIP_Real lastlowerbound;
       int parentpos;
 
       /* try to move parents downwards to insert last queue index */
-      lastlowerbound = SCIPnodeGetLowerbound(slots[lastbfsqueueidx]);
       parentpos = PQ_PARENT(freebfspos);
-      while( freebfspos > 0 && lastlowerbound < SCIPnodeGetLowerbound(slots[bfsqueue[parentpos]]) )
+      if( set->exact_enable )
       {
-         bfsqueue[freebfspos] = bfsqueue[parentpos];
-         bfsposs[bfsqueue[freebfspos]] = freebfspos;
-         freebfspos = parentpos;
-         parentpos = PQ_PARENT(freebfspos);
-         bfsparentfelldown = TRUE;
-      }
-      if( !bfsparentfelldown )
-      {
-         /* downward moving of parents was not successful -> move children upwards */
-         while( freebfspos <= PQ_PARENT(nodepq->len-1) ) /* as long as free slot has children... */
+         SCIP_RATIONAL* lastlowerbound = SCIPnodeGetLowerboundExact(slots[lastbfsqueueidx]);
+         while( freebfspos > 0 && SCIPrationalIsLT(lastlowerbound, SCIPnodeGetLowerboundExact(slots[bfsqueue[parentpos]])) )
          {
-            int childpos;
-            int brotherpos;
-
-            /* select the better child of free slot */
-            childpos = PQ_LEFTCHILD(freebfspos);
-            assert(childpos < nodepq->len);
-            brotherpos = PQ_RIGHTCHILD(freebfspos);
-            if( brotherpos < nodepq->len
-               && SCIPnodeGetLowerbound(slots[bfsqueue[brotherpos]]) < SCIPnodeGetLowerbound(slots[bfsqueue[childpos]]) )
-               childpos = brotherpos;
-
-            /* exit search loop if better child is not better than last node */
-            if( lastlowerbound <= SCIPnodeGetLowerbound(slots[bfsqueue[childpos]]) )
-               break;
-
-            /* move better child upwards, free slot is now the better child's slot */
-            bfsqueue[freebfspos] = bfsqueue[childpos];
+            bfsqueue[freebfspos] = bfsqueue[parentpos];
             bfsposs[bfsqueue[freebfspos]] = freebfspos;
-            freebfspos = childpos;
+            freebfspos = parentpos;
+            parentpos = PQ_PARENT(freebfspos);
+            bfsparentfelldown = TRUE;
+         }
+         if( !bfsparentfelldown )
+         {
+            /* downward moving of parents was not successful -> move children upwards */
+            while( freebfspos <= PQ_PARENT(nodepq->len-1) ) /* as long as free slot has children... */
+            {
+               int childpos;
+               int brotherpos;
+
+               /* select the better child of free slot */
+               childpos = PQ_LEFTCHILD(freebfspos);
+               assert(childpos < nodepq->len);
+               brotherpos = PQ_RIGHTCHILD(freebfspos);
+               if( brotherpos < nodepq->len
+                  && SCIPrationalIsLT(SCIPnodeGetLowerboundExact(slots[bfsqueue[brotherpos]]), SCIPnodeGetLowerboundExact(slots[bfsqueue[childpos]])) )
+                  childpos = brotherpos;
+
+               /* exit search loop if better child is not better than last node */
+               if( SCIPrationalIsLE(lastlowerbound, SCIPnodeGetLowerboundExact(slots[bfsqueue[childpos]])) )
+                  break;
+
+               /* move better child upwards, free slot is now the better child's slot */
+               bfsqueue[freebfspos] = bfsqueue[childpos];
+               bfsposs[bfsqueue[freebfspos]] = freebfspos;
+               freebfspos = childpos;
+            }
+         }
+      }
+      else
+      {
+         SCIP_Real lastlowerbound = SCIPnodeGetLowerbound(slots[lastbfsqueueidx]);
+         while( freebfspos > 0 && lastlowerbound < SCIPnodeGetLowerbound(slots[bfsqueue[parentpos]]) )
+         {
+            bfsqueue[freebfspos] = bfsqueue[parentpos];
+            bfsposs[bfsqueue[freebfspos]] = freebfspos;
+            freebfspos = parentpos;
+            parentpos = PQ_PARENT(freebfspos);
+            bfsparentfelldown = TRUE;
+         }
+         if( !bfsparentfelldown )
+         {
+            /* downward moving of parents was not successful -> move children upwards */
+            while( freebfspos <= PQ_PARENT(nodepq->len-1) ) /* as long as free slot has children... */
+            {
+               int childpos;
+               int brotherpos;
+
+               /* select the better child of free slot */
+               childpos = PQ_LEFTCHILD(freebfspos);
+               assert(childpos < nodepq->len);
+               brotherpos = PQ_RIGHTCHILD(freebfspos);
+               if( brotherpos < nodepq->len
+                  && SCIPnodeGetLowerbound(slots[bfsqueue[brotherpos]]) < SCIPnodeGetLowerbound(slots[bfsqueue[childpos]]) )
+                  childpos = brotherpos;
+
+               /* exit search loop if better child is not better than last node */
+               if( lastlowerbound <= SCIPnodeGetLowerbound(slots[bfsqueue[childpos]]) )
+                  break;
+
+               /* move better child upwards, free slot is now the better child's slot */
+               bfsqueue[freebfspos] = bfsqueue[childpos];
+               bfsposs[bfsqueue[freebfspos]] = freebfspos;
+               freebfspos = childpos;
+            }
          }
       }
       assert(0 <= freebfspos && freebfspos < nodepq->len);
@@ -601,6 +658,29 @@ SCIP_Real SCIPnodepqGetLowerbound(
       return SCIPsetInfinity(set);
 }
 
+/** gets the minimal exact lower bound of all nodes in the queue or NULL if empty */
+SCIP_RATIONAL* SCIPnodepqGetLowerboundExact(
+   SCIP_NODEPQ*          nodepq,             /**< node priority queue */
+   SCIP_SET*             set                 /**< global SCIP settings */
+   )
+{
+   assert(nodepq != NULL);
+   assert(nodepq->nodesel != NULL);
+   assert(set != NULL);
+
+   if( nodepq->len > 0 )
+   {
+      int bfspos;
+
+      bfspos = nodepq->bfsqueue[0];
+      assert(0 <= bfspos && bfspos < nodepq->len);
+      assert(nodepq->slots[bfspos] != NULL);
+      return SCIPnodeGetLowerboundExact(nodepq->slots[bfspos]);
+   }
+   else
+      return NULL;
+}
+
 /** gets the node with minimal lower bound of all nodes in the queue */
 SCIP_NODE* SCIPnodepqGetLowerboundNode(
    SCIP_NODEPQ*          nodepq,             /**< node priority queue */
@@ -641,8 +721,8 @@ SCIP_RETCODE SCIPnodepqBound(
    BMS_BLKMEM*           blkmem,             /**< block memory buffer */
    SCIP_SET*             set,                /**< global SCIP settings */
    SCIP_STAT*            stat,               /**< dynamic problem statistics */
-   SCIP_EVENTFILTER*     eventfilter,        /**< event filter for global (not variable dependent) events */
    SCIP_EVENTQUEUE*      eventqueue,         /**< event queue */
+   SCIP_EVENTFILTER*     eventfilter,        /**< global event filter */
    SCIP_TREE*            tree,               /**< branch and bound tree */
    SCIP_REOPT*           reopt,              /**< reoptimization data structure */
    SCIP_LP*              lp,                 /**< current LP data */
@@ -653,6 +733,7 @@ SCIP_RETCODE SCIPnodepqBound(
    int pos;
 
    assert(nodepq != NULL);
+   assert(set != NULL);
 
    SCIPsetDebugMsg(set, "bounding node queue of length %d with cutoffbound=%g\n", nodepq->len, cutoffbound);
 
@@ -666,7 +747,8 @@ SCIP_RETCODE SCIPnodepqBound(
       assert(SCIPnodeGetType(node) == SCIP_NODETYPE_LEAF);
 
       /* cut off node */
-      if( SCIPsetIsInfinity(set, SCIPnodeGetLowerbound(node)) || SCIPsetIsGE(set, SCIPnodeGetLowerbound(node), cutoffbound) )
+      if( set->exact_enable ? SCIPrationalIsGEReal(SCIPnodeGetLowerboundExact(node), cutoffbound)
+                             : SCIPsetIsGE(set, SCIPnodeGetLowerbound(node), cutoffbound) )
       {
          /* because we loop from back to front, the existing children of the node must have a smaller lower bound
           * than the cut off value
@@ -674,9 +756,11 @@ SCIP_RETCODE SCIPnodepqBound(
          assert(!node->active);
          assert(node->depth != 0 || tree->focusnode == NULL);
          assert(PQ_LEFTCHILD(pos) >= nodepq->len
-            || SCIPsetIsLT(set, SCIPnodeGetLowerbound(nodepq->slots[PQ_LEFTCHILD(pos)]), cutoffbound));
+            || SCIPsetIsLT(set, SCIPnodeGetLowerbound(nodepq->slots[PQ_LEFTCHILD(pos)]), cutoffbound)
+            || (set->exact_enable && SCIPnodeGetLowerbound(nodepq->slots[PQ_LEFTCHILD(pos)]) < cutoffbound));
          assert(PQ_RIGHTCHILD(pos) >= nodepq->len
-            || SCIPsetIsLT(set, SCIPnodeGetLowerbound(nodepq->slots[PQ_RIGHTCHILD(pos)]), cutoffbound));
+            || SCIPsetIsLT(set, SCIPnodeGetLowerbound(nodepq->slots[PQ_RIGHTCHILD(pos)]), cutoffbound)
+            || (set->exact_enable && SCIPnodeGetLowerbound(nodepq->slots[PQ_RIGHTCHILD(pos)]) < cutoffbound));
 
          SCIPsetDebugMsg(set, "cutting off leaf node in slot %d (queuelen=%d) at depth %d with lowerbound=%g\n",
             pos, SCIPnodepqLen(nodepq), SCIPnodeGetDepth(node), SCIPnodeGetLowerbound(node));
@@ -698,28 +782,67 @@ SCIP_RETCODE SCIPnodepqBound(
             --pos;
 
          node->cutoff = TRUE;
+         if( set->exact_enable )
+            SCIPrationalSetInfinity(node->lowerboundexact);
          node->lowerbound = SCIPsetInfinity(set);
          node->estimate = SCIPsetInfinity(set);
 
          if( node->depth == 0 )
             stat->rootlowerbound = SCIPsetInfinity(set);
 
-         /* update primal-dual integrals */
-         if( set->misc_calcintegral )
+         if( set->exact_enable )
          {
-            SCIP_Real lowerbound = SCIPtreeGetLowerbound(tree, set);
+            SCIP_RATIONAL* lowerboundexact = SCIPtreeGetLowerboundExact(tree, set);
 
-            assert(lowerbound <= SCIPsetInfinity(set));
+            assert(SCIPrationalIsGEReal(lowerboundexact, SCIPtreeGetLowerbound(tree, set)));
 
-            /* updating the primal integral is only necessary if lower bound has increased since last evaluation */
-            if( lowerbound > stat->lastlowerbound )
+            /* exact lower bound improved */
+            if( SCIPrationalIsLT(stat->lastlowerboundexact, lowerboundexact) )
+            {
+               /* throw improvement event if upper bound not already exceeded */
+               if( SCIPrationalIsLT(stat->lastlowerboundexact, set->scip->primal->upperboundexact) )
+               {
+                  SCIP_EVENT event;
+
+                  SCIP_CALL( SCIPeventChgType(&event, SCIP_EVENTTYPE_DUALBOUNDIMPROVED) );
+                  SCIP_CALL( SCIPeventProcess(&event, set, NULL, NULL, NULL, eventfilter) );
+               }
+
+               /* update exact last lower bound */
+               SCIPrationalSetRational(stat->lastlowerboundexact, lowerboundexact);
+            }
+         }
+
+         SCIP_Real lowerbound = SCIPtreeGetLowerbound(tree, set);
+
+         assert(lowerbound <= SCIPsetInfinity(set));
+
+         /* lower bound improved */
+         if( stat->lastlowerbound < lowerbound )
+         {
+            /* throw improvement event if not already done exactly */
+            if( !set->exact_enable && stat->lastlowerbound < set->scip->primal->upperbound )
+            {
+               SCIP_EVENT event;
+
+               SCIP_CALL( SCIPeventChgType(&event, SCIP_EVENTTYPE_DUALBOUNDIMPROVED) );
+               SCIP_CALL( SCIPeventProcess(&event, set, NULL, NULL, NULL, eventfilter) );
+            }
+
+            /* update primal-dual integrals */
+            if( set->misc_calcintegral )
+            {
                SCIPstatUpdatePrimalDualIntegrals(stat, set, set->scip->transprob, set->scip->origprob, SCIPsetInfinity(set), lowerbound);
+               assert(stat->lastlowerbound == lowerbound); /*lint !e777*/
+            }
+            else
+               stat->lastlowerbound = lowerbound;
          }
 
          SCIPvisualCutoffNode(stat->visual, set, stat, node, TRUE);
 
          /* free node memory */
-         SCIP_CALL( SCIPnodeFree(&node, blkmem, set, stat, eventfilter, eventqueue, tree, lp) );
+         SCIP_CALL( SCIPnodeFree(&node, blkmem, set, stat, eventqueue, eventfilter, tree, lp) );
       }
       else
          --pos;
@@ -732,7 +855,7 @@ SCIP_RETCODE SCIPnodepqBound(
 
 
 /*
- * node selector methods 
+ * node selector methods
  */
 
 /** method to call, when the standard mode priority of a node selector was changed */
