@@ -500,7 +500,7 @@ SCIP_Bool isNewSection(
 
       while( lpinput->token[c] != '\0' )
       {
-         token[c] = toupper(lpinput->token[c]); /*lint !e734*/
+         token[c] = toupper((unsigned char)lpinput->token[c]); /*lint !e734*/
          ++c;
          assert(c < 16);
       }
@@ -1021,6 +1021,9 @@ SCIP_RETCODE SCIPreadDiff(
 
    assert(scip != NULL);
    assert(reader != NULL);
+   assert(result != NULL);
+
+   *result = SCIP_DIDNOTRUN;
 
    /* initialize LP input data */
    lpinput.file = NULL;
