@@ -254,6 +254,9 @@ SCIP_DECL_READERREAD(readerReadSmps)
 
    assert(scip != NULL);
    assert(filename != NULL);
+   assert(result != NULL);
+
+   *result = SCIP_DIDNOTRUN;
 
    /* copy filename */
    SCIP_CALL( SCIPduplicateBufferArray(scip, &tmpfilename, filename, (int)strlen(filename)+1) );
