@@ -128,7 +128,7 @@ SCIP_Bool SCIPisCutEfficacious(
       return SCIProwIsSolEfficacious(cut, scip->set, scip->stat, sol, (SCIPtreeGetCurrentDepth(scip->tree) == 0));
 }
 
-/** checks, if the given cut's efficacy is larger than the minimal cut efficacy
+/** checks if the given cut's efficacy is larger than the minimal cut efficacy
  *
  *  @return TRUE if the given cut's efficacy is larger than the minimal cut efficacy, otherwise FALSE
  */
@@ -214,32 +214,6 @@ SCIP_Bool SCIPisCutApplicable(
    return SCIPsepastoreIsCutApplicable(scip->set, cut);
 }
 
-/** adds cut to separation storage
- *
- *  @return \ref SCIP_OKAY is returned if everything worked. Otherwise a suitable error code is passed. See \ref
- *          SCIP_Retcode "SCIP_RETCODE" for a complete list of error codes.
- *
- *  @pre This method can be called if @p scip is in one of the following stages:
- *       - \ref SCIP_STAGE_SOLVING
- *
- *  @deprecated Please use SCIPaddRow() instead, or, if the row is a global cut, add it only to the global cutpool.
- */
-SCIP_DEPRECATED
-SCIP_RETCODE SCIPaddCut(
-   SCIP*                 scip,               /**< SCIP data structure */
-   SCIP_SOL*             sol,                /**< primal solution that was separated, or NULL for LP solution */
-   SCIP_ROW*             cut,                /**< separated cut */
-   SCIP_Bool             forcecut,           /**< should the cut be forced to enter the LP? */
-   SCIP_Bool*            infeasible          /**< pointer to store whether cut has been detected to be infeasible for local bounds */
-   )
-{
-   SCIP_CALL( SCIPcheckStage(scip, "SCIPaddCut", FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, TRUE, FALSE, FALSE, FALSE, FALSE) );
-
-   SCIP_UNUSED(sol);
-
-   return SCIPaddRow(scip, cut, forcecut, infeasible);
-}
-
 /** adds row to separation storage
  *
  *  @return \ref SCIP_OKAY is returned if everything worked. Otherwise a suitable error code is passed. See \ref
