@@ -231,7 +231,7 @@ SCIP_RETCODE parseBase(
       ++expr;
       debugParse("Done parsing expression, continue with <%s>\n", expr);
    }
-   else if( isdigit(*expr) )
+   else if( isdigit((unsigned char)*expr) )
    {
       /* parse number */
       SCIP_Real value;
@@ -241,9 +241,14 @@ SCIP_RETCODE parseBase(
          return SCIP_READERROR;
       }
       debugParse("Parsed value %g, creating a value-expression.\n", value);
+      if( !SCIPisFinite(value) || SCIPisInfinity(scip, value) )
+      {
+         SCIPerrorMessage("Infinite or nan term value expression within nonlinear expression %s\n", expr);
+         return SCIP_INVALIDDATA;
+      }
       SCIP_CALL( SCIPcreateExprValue(scip, basetree, value, ownercreate, ownercreatedata) );
    }
-   else if( isalpha(*expr) )
+   else if( isalpha((unsigned char)*expr) )
    {
       /* a (function) name is coming, should find exprhandler with such name */
       int i;
@@ -253,7 +258,7 @@ SCIP_RETCODE parseBase(
 
       /* get name */
       i = 0;
-      while( *expr != '(' && *expr != '\0' && !isspace(*expr)
+      while( *expr != '(' && *expr != '\0' && !isspace((unsigned char)*expr)
              && !( *expr == '\\' && *(expr+1) != '\0' && strchr(SCIP_SPACECONTROL, *(expr+1)) ) )
       {
          operatorname[i] = *expr;
@@ -386,7 +391,7 @@ SCIP_RETCODE parseFactor(
          /* no parenthesis, we should see just a positive number */
 
          /* expect a digit */
-         if( isdigit(*expr) )
+         if( isdigit((unsigned char)*expr) )
          {
             if( !SCIPstrToRealValue(expr, &exponent, (char**)&expr) )
             {
@@ -403,6 +408,13 @@ SCIP_RETCODE parseFactor(
          }
       }
 
+      if( !SCIPisFinite(exponent) || SCIPisInfinity(scip, exponent) )
+      {
+         SCIPerrorMessage("Infinite or nan term exponent in nonlinear expression %s\n", expr);
+         SCIP_CALL( SCIPreleaseExpr(scip, &basetree) );
+         return SCIP_INVALIDDATA;
+      }
+
       debugParse("parsed the exponent %g\n", exponent); /*lint !e506 !e681*/
    }
    else
@@ -572,6 +584,13 @@ SCIP_RETCODE parseExpr(
          /* check if we have a "coef * <term>" */
          if( SCIPstrToRealValue(expr, &coef, (char**)newpos) )
          {
+            if( !SCIPisFinite(coef) || SCIPisInfinity(scip, coef) )
+            {
+               SCIPerrorMessage("Infinite or nan term coefficient in nonlinear expression %s\n", expr);
+               SCIP_CALL( SCIPreleaseExpr(scip, exprtree) );
+               return SCIP_INVALIDDATA;
+            }
+
             SCIP_CALL( SCIPskipSpace((char**)newpos) );
 
             if( **newpos == '<' )
@@ -2016,7 +2035,7 @@ SCIP_RETCODE SCIPcomputeExprCurvature(
 
 /** computes integrality information of a given expression and all its subexpressions
  *
- * The integrality information can be accessed via SCIPexprIsIntegral().
+ * The integrality information can be accessed via SCIPexprGetIntegrality() and SCIPexprIsIntegral()
  */
 SCIP_RETCODE SCIPcomputeExprIntegrality(
    SCIP*                 scip,               /**< SCIP data structure */
@@ -2024,7 +2043,7 @@ SCIP_RETCODE SCIPcomputeExprIntegrality(
    )
 {
    SCIP_EXPRITER* it;
-   SCIP_Bool isintegral;
+   SCIP_IMPLINTTYPE integrality;
 
    assert(scip != NULL);
    assert(scip->mem != NULL);
@@ -2034,8 +2053,8 @@ SCIP_RETCODE SCIPcomputeExprIntegrality(
    if( SCIPexprGetNChildren(expr) == 0 )
    {
       /* compute integrality information */
-      SCIP_CALL( SCIPexprhdlrIntegralityExpr(SCIPexprGetHdlr(expr), scip->set, expr, &isintegral) );
-      SCIPexprSetIntegrality(expr, isintegral);
+      SCIP_CALL( SCIPexprhdlrIntegralityExpr(SCIPexprGetHdlr(expr), scip->set, expr, &integrality) );
+      SCIPexprSetIntegrality(expr, integrality);
 
       return SCIP_OKAY;
    }
@@ -2047,8 +2066,8 @@ SCIP_RETCODE SCIPcomputeExprIntegrality(
    for( expr = SCIPexpriterGetCurrent(it); !SCIPexpriterIsEnd(it); expr = SCIPexpriterGetNext(it) )
    {
       /* compute integrality information */
-      SCIP_CALL( SCIPexprhdlrIntegralityExpr(SCIPexprGetHdlr(expr), scip->set, expr, &isintegral) );
-      SCIPexprSetIntegrality(expr, isintegral);
+      SCIP_CALL( SCIPexprhdlrIntegralityExpr(SCIPexprGetHdlr(expr), scip->set, expr, &integrality) );
+      SCIPexprSetIntegrality(expr, integrality);
    }
 
    SCIPexpriterFree(&it);
