@@ -397,8 +397,8 @@ SCIP_RETCODE SCIPapplyReopt(
    SCIP_CALL( SCIPcheckStage(scip, "SCIPapplyReopt", FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, TRUE, TRUE, FALSE, FALSE, FALSE) );
 
    SCIP_CALL( SCIPreoptApply(scip->reopt, scip, scip->set, scip->stat, scip->transprob, scip->origprob, scip->tree,
-         scip->lp, scip->branchcand, scip->eventqueue, scip->cliquetable, scip->mem->probmem, reoptnode, id, estimate,
-         childnodes, ncreatedchilds, naddedconss, childnodessize, success) );
+         scip->lp, scip->branchcand, scip->eventqueue, scip->eventfilter, scip->cliquetable, scip->mem->probmem,
+         reoptnode, id, estimate, childnodes, ncreatedchilds, naddedconss, childnodessize, success) );
 
    return SCIP_OKAY;
 }
