@@ -1126,7 +1126,8 @@ SCIP_RETCODE buildMod2Matrix(
       int rhsmod2;
 
       /* skip non-integral rows and rows not suitable for generating cuts */
-      if( SCIProwIsModifiable(rows[i]) || !SCIProwIsIntegral(rows[i]) || (SCIProwIsLocal(rows[i]) && !allowlocal) || SCIProwGetNNonz(rows[i]) > maxnonzeros )
+      if( SCIProwIsModifiable(rows[i]) || !SCIProwIsIntegral(rows[i]) ||
+            (SCIProwIsLocal(rows[i]) && !allowlocal) || SCIProwGetNNonz(rows[i]) > maxnonzeros )
          continue;
 
       lhsmod2 = 0;
