@@ -38,6 +38,7 @@
 #include "scip/set.h"
 #include "scip/stat.h"
 #include "scip/lp.h"
+#include "scip/lpexact.h"
 #include "scip/var.h"
 #include "scip/tree.h"
 #include "scip/reopt.h"
@@ -52,6 +53,8 @@
 #include "scip/struct_event.h"
 #include "scip/struct_sepastore.h"
 #include "scip/misc.h"
+#include "scip/pub_lpexact.h"
+#include "scip/scip_lpexact.h"
 
 
 
@@ -195,6 +198,10 @@ SCIP_Bool sepastoreIsCutRedundant(
    if( SCIProwIsModifiable(cut) )
       return FALSE;
 
+   /** @todo implement a safe redundancy check for cuts in exact solving mode */
+   if( set->exact_enable )
+      return FALSE;
+
    /* check for activity redundancy */
    lhs = SCIProwGetLhs(cut);
    rhs = SCIProwGetRhs(cut);
@@ -234,6 +241,10 @@ SCIP_Bool sepastoreIsCutRedundantOrInfeasible(
 
    *infeasible = FALSE;
 
+   /** @todo implement a safe redundancy or infeasibility check for cuts in exact solving mode */
+   if( set->exact_enable )
+      return FALSE;
+
    /* modifiable cuts cannot be declared redundant or infeasible, since we don't know all coefficients */
    if( SCIProwIsModifiable(cut) )
       return FALSE;
@@ -593,6 +604,7 @@ SCIP_RETCODE sepastoreApplyLb(
    SCIP_LP*              lp,                 /**< LP data */
    SCIP_BRANCHCAND*      branchcand,         /**< branching candidate storage */
    SCIP_EVENTQUEUE*      eventqueue,         /**< event queue */
+   SCIP_EVENTFILTER*     eventfilter,        /**< global event filter */
    SCIP_CLIQUETABLE*     cliquetable,        /**< clique table data structure */
    SCIP_VAR*             var,                /**< problem variable */
    SCIP_Real             bound,              /**< new lower bound of variable */
@@ -622,7 +634,7 @@ SCIP_RETCODE sepastoreApplyLb(
          if( !SCIPsetIsInfinity(set, bound) && SCIPsetIsFeasLE(set, bound, SCIPvarGetUbLocal(var)) )
          {
             SCIP_CALL( SCIPnodeAddBoundchg(SCIPtreeGetCurrentNode(tree), blkmem, set, stat, transprob, origprob, tree,
-                  reopt, lp, branchcand, eventqueue, cliquetable, var, bound, SCIP_BOUNDTYPE_LOWER, FALSE) );
+                  reopt, lp, branchcand, eventqueue, eventfilter, cliquetable, var, bound, SCIP_BOUNDTYPE_LOWER, FALSE) );
          }
          else
             *cutoff = TRUE;
@@ -649,12 +661,12 @@ SCIP_RETCODE sepastoreApplyLb(
          if( !SCIPsetIsInfinity(set, bound) && SCIPsetIsFeasLE(set, bound, SCIPvarGetUbGlobal(var)) )
          {
             SCIP_CALL( SCIPnodeAddBoundchg(SCIPtreeGetRootNode(tree), blkmem, set, stat, transprob, origprob, tree, reopt,
-                  lp, branchcand, eventqueue, cliquetable, var, bound, SCIP_BOUNDTYPE_LOWER, FALSE) );
+                  lp, branchcand, eventqueue, eventfilter, cliquetable, var, bound, SCIP_BOUNDTYPE_LOWER, FALSE) );
          }
          else
          {
             /* we are done with solving since a global bound change is infeasible */
-            SCIP_CALL( SCIPnodeCutoff(SCIPtreeGetRootNode(tree), set, stat, tree, transprob, origprob, reopt, lp, blkmem) );
+            SCIP_CALL( SCIPnodeCutoff(SCIPtreeGetRootNode(tree), set, stat, eventfilter, tree, transprob, origprob, reopt, lp, blkmem) );
             *cutoff = TRUE;
          }
 
@@ -684,6 +696,7 @@ SCIP_RETCODE sepastoreApplyUb(
    SCIP_LP*              lp,                 /**< LP data */
    SCIP_BRANCHCAND*      branchcand,         /**< branching candidate storage */
    SCIP_EVENTQUEUE*      eventqueue,         /**< event queue */
+   SCIP_EVENTFILTER*     eventfilter,        /**< global event filter */
    SCIP_CLIQUETABLE*     cliquetable,        /**< clique table data structure */
    SCIP_VAR*             var,                /**< problem variable */
    SCIP_Real             bound,              /**< new upper bound of variable */
@@ -713,7 +726,7 @@ SCIP_RETCODE sepastoreApplyUb(
          if( !SCIPsetIsInfinity(set, -bound) && SCIPsetIsFeasGE(set, bound, SCIPvarGetLbLocal(var)) )
          {
             SCIP_CALL( SCIPnodeAddBoundchg(SCIPtreeGetCurrentNode(tree), blkmem, set, stat, transprob, origprob, tree,
-                  reopt, lp, branchcand, eventqueue, cliquetable, var, bound, SCIP_BOUNDTYPE_UPPER, FALSE) );
+                  reopt, lp, branchcand, eventqueue, eventfilter, cliquetable, var, bound, SCIP_BOUNDTYPE_UPPER, FALSE) );
          }
          else
             *cutoff = TRUE;
@@ -740,12 +753,12 @@ SCIP_RETCODE sepastoreApplyUb(
          if( !SCIPsetIsInfinity(set, -bound) && SCIPsetIsFeasGE(set, bound, SCIPvarGetLbGlobal(var)) )
          {
             SCIP_CALL( SCIPnodeAddBoundchg(SCIPtreeGetRootNode(tree), blkmem, set, stat, transprob, origprob, tree, reopt,
-                  lp, branchcand, eventqueue, cliquetable, var, bound, SCIP_BOUNDTYPE_UPPER, FALSE) );
+                  lp, branchcand, eventqueue, eventfilter, cliquetable, var, bound, SCIP_BOUNDTYPE_UPPER, FALSE) );
          }
          else
          {
             /* we are done with solving since a global bound change is infeasible */
-            SCIP_CALL( SCIPnodeCutoff(SCIPtreeGetRootNode(tree), set, stat, tree, transprob, origprob, reopt, lp, blkmem) );
+            SCIP_CALL( SCIPnodeCutoff(SCIPtreeGetRootNode(tree), set, stat, eventfilter, tree, transprob, origprob, reopt, lp, blkmem) );
             *cutoff = TRUE;
          }
 
@@ -775,6 +788,7 @@ SCIP_RETCODE sepastoreApplyBdchg(
    SCIP_LP*              lp,                 /**< LP data */
    SCIP_BRANCHCAND*      branchcand,         /**< branching candidate storage */
    SCIP_EVENTQUEUE*      eventqueue,         /**< event queue */
+   SCIP_EVENTFILTER*     eventfilter,        /**< global event filter */
    SCIP_CLIQUETABLE*     cliquetable,        /**< clique table data structure */
    SCIP_ROW*             cut,                /**< cut with a single variable */
    SCIP_Bool*            applied,            /**< pointer to store whether the domain change was applied */
@@ -821,13 +835,13 @@ SCIP_RETCODE sepastoreApplyBdchg(
       {
          /* coefficient is positive -> lhs corresponds to lower bound */
          SCIP_CALL( sepastoreApplyLb(sepastore, blkmem, set, stat, transprob, origprob, tree, reopt, lp, branchcand, eventqueue,
-               cliquetable, var, lhs/vals[0], local, applied, cutoff) );
+               eventfilter, cliquetable, var, lhs/vals[0], local, applied, cutoff) );
       }
       else
       {
          /* coefficient is negative -> lhs corresponds to upper bound */
          SCIP_CALL( sepastoreApplyUb(sepastore, blkmem, set, stat, transprob, origprob, tree, reopt, lp, branchcand, eventqueue,
-               cliquetable, var, lhs/vals[0], local, applied, cutoff) );
+               eventfilter, cliquetable, var, lhs/vals[0], local, applied, cutoff) );
       }
    }
 
@@ -840,13 +854,13 @@ SCIP_RETCODE sepastoreApplyBdchg(
       {
          /* coefficient is positive -> rhs corresponds to upper bound */
          SCIP_CALL( sepastoreApplyUb(sepastore, blkmem, set, stat, transprob, origprob, tree, reopt, lp, branchcand, eventqueue,
-               cliquetable, var, rhs/vals[0], local, applied, cutoff) );
+               eventfilter, cliquetable, var, rhs/vals[0], local, applied, cutoff) );
       }
       else
       {
          /* coefficient is negative -> rhs corresponds to lower bound */
          SCIP_CALL( sepastoreApplyLb(sepastore, blkmem, set, stat, transprob, origprob, tree, reopt, lp, branchcand, eventqueue,
-               cliquetable, var, rhs/vals[0], local, applied, cutoff) );
+               eventfilter, cliquetable, var, rhs/vals[0], local, applied, cutoff) );
       }
    }
 
@@ -999,16 +1013,36 @@ SCIP_RETCODE SCIPsepastoreApplyCuts(
          SCIP_Bool applied = FALSE;
 
          /* if the cut is a bound change (i.e. a row with only one variable), add it as bound change instead of LP row */
-         if( !SCIProwIsModifiable(cut) && SCIProwGetNNonz(cut) == 1 )
+         if( !SCIProwIsModifiable(cut) && SCIProwGetNNonz(cut) == 1 && !set->exact_enable )
          {
             SCIPsetDebugMsg(set, " -> applying forced cut <%s> as boundchange\n", SCIProwGetName(cut));
             SCIP_CALL( sepastoreApplyBdchg(sepastore, blkmem, set, stat, transprob, origprob, tree, reopt, lp, branchcand,
-                  eventqueue, cliquetable, cut, &applied, cutoff) );
+                  eventqueue, eventfilter, cliquetable, cut, &applied, cutoff) );
             assert(applied || !sepastoreIsBdchgApplicable(set, cut));
          }
 
          if( !applied )
          {
+            /* create rational representation of the cut; note that this may slightly change the floating-point
+             * coefficients
+             */
+            if( set->exact_enable && SCIProwGetRowExact(cut) == NULL )
+            {
+               SCIP_Bool poolcut = FALSE;
+               if( !SCIProwIsLocal(cut) && !(SCIPisCutNew(set->scip, cut)) )
+               {
+                  poolcut = TRUE;
+                  SCIP_CALL( SCIPdelPoolCut(set->scip, cut) );
+               }
+               SCIP_CALL( SCIProwExactCreateFromRow(cut, blkmem, set, stat, eventqueue, transprob, lp->lpexact) );
+               if( poolcut )
+               {
+                  SCIP_CALL( SCIPaddPoolCut(set->scip, cut) );
+               }
+
+               SCIP_CALL( SCIPaddRowExact(set->scip, cut->rowexact) );
+            }
+
             /* add cut to the LP and update orthogonalities */
             SCIPsetDebugMsg(set, " -> applying%s cut <%s>\n", (i < sepastore->nforcedcuts) ? " forced" : "", SCIProwGetName(cut));
             /*SCIPdebug( SCIProwPrint(cut, set->scip->messagehdlr, NULL));*/
