@@ -142,6 +142,7 @@ struct SCIP_Node
 {
    SCIP_Longint          number;             /**< successively assigned number of the node */
    SCIP_Real             lowerbound;         /**< lower (dual) bound of subtree */
+   SCIP_RATIONAL*        lowerboundexact;    /**< lower (dual) rational bound of subtree */
    SCIP_Real             estimate;           /**< estimated value of feasible solution in subtree */
    union
    {
@@ -173,6 +174,7 @@ struct SCIP_PendingBdchg
    SCIP_NODE*            node;               /**< node to add bound change to */
    SCIP_VAR*             var;                /**< variable to change the bounds for */
    SCIP_Real             newbound;           /**< new value for bound */
+   SCIP_RATIONAL*        newboundexact;      /**< new value for exact bound, or NULL if not needed */
    SCIP_BOUNDTYPE        boundtype;          /**< type of bound: lower or upper bound */
    SCIP_CONS*            infercons;          /**< constraint that deduced the bound change, or NULL */
    SCIP_PROP*            inferprop;          /**< propagator that deduced the bound change, or NULL */
@@ -212,6 +214,7 @@ struct SCIP_Tree
    SCIP_LPINORMS*        probinglpinorms;    /**< LP pricing norms information before probing started */
    SCIP_PENDINGBDCHG*    pendingbdchgs;      /**< array of pending bound changes, or NULL */
    SCIP_Real*            probdiverelaxsol;   /**< array with stored original relaxation solution during diving or probing */
+   SCIP_Real             probinglpobjval;    /**< lp objective value before probing started */
    int                   nprobdiverelaxsol;  /**< size of probdiverelaxsol */
    SCIP_Longint          focuslpstateforklpcount; /**< LP number of last solved LP in current LP state fork, or -1 if unknown */
    SCIP_Longint          lastbranchparentid; /**< last node id/number of branching parent */
@@ -248,6 +251,7 @@ struct SCIP_Tree
    SCIP_Bool             probinglpwasprimchecked;/**< primal feasibility has been checked when probing started */
    SCIP_Bool             probinglpwasdualfeas;/**< dual feasibility when probing started */
    SCIP_Bool             probinglpwasdualchecked;/**< dual feasibility has been check when probing started */
+   SCIP_Bool             probinglphadsafebound;/**< lp had a proven dual bound when probing started? */
    SCIP_Bool             probdiverelaxstored; /**< was a relax solution stored before diving or probing ? */
    SCIP_Bool             probdiverelaxincludeslp; /**< did the stored relaxation solution include all lp cuts ? */
 };
